@@ -4,7 +4,7 @@
  *
  *  This module implements the DEFINE text function.
  *
- *  Time-stamp:        "2012-02-12 09:01:11 bkorb"
+ *  Time-stamp:        "2012-04-07 09:47:37 bkorb"
  *
  *  This file is part of AutoGen.
  *  AutoGen Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
@@ -25,9 +25,9 @@
 
 typedef int (tCmpProc)(const void*, const void*);
 
-typedef struct def_list tDefList;
+typedef struct def_list def_list_t;
 struct def_list {
-    tDefEntry  de;
+    def_ent_t  de;
     char*      pzExpr;
 };
 
@@ -35,87 +35,93 @@ struct def_list {
 static int
 order_def_list(const void* p1, const void* p2);
 
-static tDefList *
-link_twins(tDefList* pDL, tDefList* pNext, int* pCt);
+static def_list_t *
+link_twins(def_list_t * pDL, def_list_t * pNext, int* pCt);
 
 static uint_t
-count_named_values(tTemplate* pT, tMacro* pMac);
+count_named_values(templ_t * pT, macro_t * mac);
 
 static char *
-gather_assigned_value(char * pzScan, tDefList * pDL);
+gather_assigned_value(char * pzScan, def_list_t * pDL);
 
 static void
-fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac);
+fill_in_values(def_list_t * pDL, char * pzScan, templ_t * pT, macro_t * mac);
 
 static void
-prep_invoke_args(tMacro * pMac);
+prep_invoke_args(macro_t * mac);
 
 static void
-build_defs(int defCt, tDefList* pList);
+build_defs(int def_ct, def_list_t * def_list);
+
+static templ_t *
+new_template(templ_t * base_tpl, macro_t * mac, char const * scan);
+
+static void
+load_define_tpl(templ_t * tpl, char const ** ppzScan);
 /* = = = END-STATIC-FORWARD = = = */
 
 static int
 order_def_list(const void* p1, const void* p2)
 {
-    tDefEntry * pDL1 = (tDefEntry*)p1;
-    tDefEntry * pDL2 = (tDefEntry*)p2;
-    int cmp = streqvcmp(pDL1->pzDefName, pDL2->pzDefName);
+    def_ent_t * pDL1 = (def_ent_t*)p1;
+    def_ent_t * pDL2 = (def_ent_t*)p2;
+    int cmp = streqvcmp(pDL1->de_name, pDL2->de_name);
 
     /*
      *  IF the names are the same, then we order them based on which name
      *  appears first.  Do not reorder entries with the same name.
      */
     if (cmp == 0)
-        cmp = (int)(pDL1->pzDefName - pDL2->pzDefName);
+        cmp = (int)(pDL1->de_name - pDL2->de_name);
     return cmp;
 }
 
-static tDefList *
-link_twins(tDefList* pDL, tDefList* pNext, int* pCt)
+static def_list_t *
+link_twins(def_list_t * pDL, def_list_t * pNext, int* pCt)
 {
-    tDefList* pN;
+    def_list_t * pN;
     int  ct  = *pCt;
     int  idx = 1;
 
-    pDL->de.pTwin = &(pNext->de);
-    pNext->de.pPrevTwin = &(pDL->de);
+    pDL->de.de_twin = &(pNext->de);
+    pNext->de.de_ptwin = &(pDL->de);
 
     for (;;) {
-        pNext->de.index = idx++;
+        pNext->de.de_index = idx++;
         pN = pNext + 1; /* We return this, valid or not */
         if (--ct <= 0)  /* count each successive twin   */
             break;
-        if (streqvcmp(pNext->de.pzDefName, pN->de.pzDefName) != 0)
+        if (streqvcmp(pNext->de.de_name, pN->de.de_name) != 0)
             break;
 
         /*
          *  We have found another twin.  Link it in and advance
          */
-        pNext->de.pTwin  = &(pN->de);
-        pN->de.pPrevTwin = &(pNext->de);
+        pNext->de.de_twin  = &(pN->de);
+        pN->de.de_ptwin = &(pNext->de);
         pNext = pN;
     }
 
-    pDL->de.pEndTwin  = &(pNext->de);
-    pNext->de.pTwin   = NULL; /* NULL terminated list */
-    pDL->de.pPrevTwin = NULL; /* NULL terminated list */
+    pDL->de.de_etwin  = &(pNext->de);
+    pNext->de.de_twin = NULL; /* NULL terminated list */
+    pDL->de.de_ptwin  = NULL; /* NULL terminated list */
     *pCt = ct;
-    pDL->de.pNext = NULL;     /* in case ct == 0      */
+    pDL->de.de_next   = NULL; /* in case ct == 0      */
     return pN; /* If ct is zero, then this is invalid */
 }
 
 
 static uint_t
-count_named_values(tTemplate* pT, tMacro* pMac)
+count_named_values(templ_t * pT, macro_t * mac)
 {
-    char * pzScan = pT->pzTemplText + pMac->ozText;
+    char * pzScan = pT->td_text + mac->md_txt_off;
     uint_t ct = 0;
 
     while (*pzScan != NUL) {
         ct++;
         if (! IS_VAR_FIRST_CHAR(*pzScan)) {
             fprintf(stderr, NAMED_VALUES_WHERE, ct, pzScan);
-            AG_ABEND_IN(pT, pMac, NAMED_VALUES_NO_NAME);
+            AG_ABEND_IN(pT, mac, NAMED_VALUES_NO_NAME);
         }
 
         pzScan = SPN_VALUE_NAME_CHARS(pzScan);
@@ -123,7 +129,7 @@ count_named_values(tTemplate* pT, tMacro* pMac)
         if (*pzScan != '=')
             continue;
         pzScan = SPN_WHITESPACE_CHARS(pzScan+1);
-        pzScan = (char*)skipExpression(pzScan, strlen(pzScan));
+        pzScan = (char*)skip_expr(pzScan, strlen(pzScan));
         pzScan = SPN_WHITESPACE_CHARS(pzScan);
     }
 
@@ -132,13 +138,13 @@ count_named_values(tTemplate* pT, tMacro* pMac)
 
 
 static char *
-gather_assigned_value(char * pzScan, tDefList * pDL)
+gather_assigned_value(char * pzScan, def_list_t * pDL)
 {
     pzScan = SPN_WHITESPACE_CHARS(pzScan);
-    strtransform(pDL->de.pzDefName, pDL->de.pzDefName);
+    strtransform(pDL->de.de_name, pDL->de.de_name);
     pDL->pzExpr = pzScan;
-    pDL->de.valType = VALTYP_TEXT;
-    pzScan = (char*)skipExpression(pzScan, strlen(pzScan));
+    pDL->de.de_type = VALTYP_TEXT;
+    pzScan = (char*)skip_expr(pzScan, strlen(pzScan));
 
     /*
      *  Figure out what kind of expression we have
@@ -158,7 +164,7 @@ gather_assigned_value(char * pzScan, tDefList * pDL)
          *  Process the quoted string, but leave a '`' marker, too
          */
         AGDUPSTR(pz, pDL->pzExpr, "macro arg expr");
-        spanQuote(pz);
+        span_quote(pz);
         strcpy(pDL->pzExpr+1, pz);
         AGFREE((void*)pz);
         break;
@@ -172,16 +178,15 @@ gather_assigned_value(char * pzScan, tDefList * pDL)
             char* pz = (char*)AGALOC(24, "quoted string");
             memcpy((void*)pz, pDL->pzExpr, (size_t)(pzScan - pDL->pzExpr));
             pDL->pzExpr = pz;
-            manageAllocatedData(pz);
         }
-        spanQuote(pDL->pzExpr);
+        span_quote(pDL->pzExpr);
         /* FALLTHROUGH */
 
     default:
         /*
          *  Default:  the raw sequence of characters is the value
          */
-        pDL->de.val.pzText = pDL->pzExpr;
+        pDL->de.de_val.dvu_text = pDL->pzExpr;
         pDL->pzExpr        = NULL;
     }
 
@@ -190,19 +195,19 @@ gather_assigned_value(char * pzScan, tDefList * pDL)
 
 
 static void
-fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
+fill_in_values(def_list_t * pDL, char * pzScan, templ_t * pT, macro_t * mac)
 {
     for (;; pDL++ ) {
-        pDL->de.pzDefName = pzScan;
+        pDL->de.de_name = pzScan;
         pzScan = SPN_VALUE_NAME_CHARS(pzScan);
 
         switch (*pzScan) {
         case NUL:
-            pDL->de.val.pzText = (char*)zNil;
+            pDL->de.de_val.dvu_text = (char*)zNil;
             return;
 
         default:
-            AG_ABEND_IN(pT, pMac, FILL_IN_VAL_NO_ASSIGN);
+            AG_ABEND_IN(pT, mac, FILL_IN_VAL_NO_ASSIGN);
 
         case ' ': case TAB: case NL: case '\f':
             *(pzScan++) = NUL;
@@ -212,7 +217,7 @@ fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
              *  The name was separated by space, but has no value
              */
             if (*pzScan != '=') {
-                pDL->de.val.pzText = (char*)zNil;
+                pDL->de.de_val.dvu_text = (char*)zNil;
                 if (*pzScan == NUL)
                     return;
                 continue;
@@ -237,7 +242,7 @@ fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
             break;
 
         if (! IS_WHITESPACE_CHAR(*pzScan))
-            AG_ABEND_IN(pT, pMac, FILL_IN_VAL_NO_SEP);
+            AG_ABEND_IN(pT, mac, FILL_IN_VAL_NO_SEP);
 
         /*
          *  Terminate the string value and skip over any additional space
@@ -248,7 +253,7 @@ fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
 }
 
 /*
- *  parseMacroArgs
+ *  parse_mac_args
  *
  *  This routine is called just before the first call to mFunc_Define
  *  for a particular macro invocation.  It scans the text of the invocation
@@ -256,45 +261,45 @@ fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
  *  of the processing of the user defined macro.
  */
 LOCAL void
-parseMacroArgs(tTemplate* pT, tMacro* pMac)
+parse_mac_args(templ_t * pT, macro_t * mac)
 {
-    char *      pzScan = pT->pzTemplText + pMac->ozText;
-    uint_t      ct;
-    tDefList *  pDL;
-    tDefList *  pN;
+    char *       pzScan = pT->td_text + mac->md_txt_off;
+    uint_t       ct;
+    def_list_t * pDL;
+    def_list_t * pN;
 
     /*
      *  If there is no argument text, then the arg count is zero.
      */
-    if (pMac->ozText == 0) {
-        pMac->res = 0;
+    if (mac->md_txt_off == 0) {
+        mac->md_res = 0;
         return;
     }
 
-    ct = count_named_values(pT, pMac);
+    ct = count_named_values(pT, mac);
 
     /*
      *  The result is zero if we don't have any
      */
-    pMac->sibIndex = ct;
+    mac->md_sib_idx = ct;
     if (ct == 0) {
-        pMac->ozText = 0;
-        pMac->res = 0;
+        mac->md_txt_off = 0;
+        mac->md_res = 0;
         return;
     }
 
     /*
      *  Allocate the array of definition descriptors
      */
-    pzScan = pT->pzTemplText + pMac->ozText;
-    pDL = (tDefList*)AGALOC(ct * sizeof(tDefList), "array of def desc");
-    memset((void*)pDL, 0, ct * sizeof(tDefList));
-    pMac->res = (uintptr_t)pDL;
+    pzScan = pT->td_text + mac->md_txt_off;
+    pDL = (def_list_t *)AGALOC(ct * sizeof(def_list_t), "array of def desc");
+    memset((void*)pDL, 0, ct * sizeof(def_list_t));
+    mac->md_res = (uintptr_t)pDL;
 
     /*
      *  Fill in the array of value assignments
      */
-    fill_in_values(pDL, pzScan, pT, pMac);
+    fill_in_values(pDL, pzScan, pT, mac);
 
     if (ct > 1) {
         /*
@@ -303,15 +308,15 @@ parseMacroArgs(tTemplate* pT, tMacro* pMac)
          *  Order is preserved by comparing string addresses,
          *  if the strings compare equal.
          */
-        pDL = (tDefList*)pMac->res;
-        qsort((void*)pDL, (size_t)ct, sizeof(tDefList), order_def_list);
+        pDL = (def_list_t *)mac->md_res;
+        qsort((void*)pDL, (size_t)ct, sizeof(def_list_t), order_def_list);
 
         /*
          *  Now, link them all together.  Singly linked list.
          */
         for (;;) {
             if (--ct == 0) {
-                pDL->de.pNext = NULL;
+                pDL->de.de_next = NULL;
                 break;
             }
 
@@ -321,37 +326,47 @@ parseMacroArgs(tTemplate* pT, tMacro* pMac)
              *  IF the next entry has the same name,
              *  THEN it is a "twin".  Link twins on the twin list.
              */
-            if (streqvcmp(pDL->de.pzDefName, pN->de.pzDefName) == 0) {
+            if (streqvcmp(pDL->de.de_name, pN->de.de_name) == 0) {
                 pN = link_twins(pDL, pN, (int*)&ct);
                 if (ct <= 0)
                     break;  /* pN is now invalid */
             }
 
-            pDL->de.pNext = &(pN->de);
+            pDL->de.de_next = &(pN->de);
             pDL = pN;
         }
     }
 }
 
+/**
+ * prepare the args for INVOKE macro.
+ * See if there's any text following the name of the DEFINE macro to invoke.
+ * If there is, then set up the arguments now so it's easy to deal with
+ * next time around.  The name of the macro to invoke may be dynamic.
+ * "skip_expr" skips over either a name or a scheme expression that
+ * is supposed to yield a name.
+ *
+ * @param mac  the macro structure describing the invocation
+ */
 static void
-prep_invoke_args(tMacro * pMac)
+prep_invoke_args(macro_t * mac)
 {
     char * pzText;
-    tTemplate * pT = pCurTemplate;
+    templ_t * pT = current_tpl;
 
-    if (pMac->ozText == 0)
-        AG_ABEND_IN(pT, pMac, PREP_INVOKE_NO_NAME);
-    pMac->ozName = pMac->ozText;
-    pzText = pT->pzTemplText + pMac->ozText;
-    pzText = (char*)skipExpression(pzText, strlen(pzText));
+    if (mac->md_txt_off == 0)
+        AG_ABEND_IN(pT, mac, PREP_INVOKE_NO_NAME);
+    mac->md_name_off = mac->md_txt_off;
+    pzText = pT->td_text + mac->md_txt_off;
+    pzText = (char*)skip_expr(pzText, strlen(pzText));
 
     /*
      *  IF there is no more text,
      *  THEN there are no arguments
      */
     if (*pzText == NUL) {
-        pMac->ozText = 0;
-        pMac->res = 0;
+        mac->md_txt_off = 0;
+        mac->md_res = 0;
     }
 
     /*
@@ -360,16 +375,15 @@ prep_invoke_args(tMacro * pMac)
      */
     else {
         if (! IS_WHITESPACE_CHAR(*pzText))
-            AG_ABEND_IN(pT, pMac, PREP_INVOKE_NO_SEP);
+            AG_ABEND_IN(pT, mac, PREP_INVOKE_NO_SEP);
         *pzText = NUL;
         pzText = SPN_WHITESPACE_CHARS(pzText + 1);
-        pMac->ozText = pzText - pT->pzTemplText;
-        parseMacroArgs(pT, pMac);
-        pCurTemplate = pT;
+        mac->md_txt_off = pzText - pT->td_text;
+        parse_mac_args(pT, mac);
+        current_tpl = pT;
     }
 }
 
-
 /*=macfunc DEBUG
  *
  *  handler-proc:
@@ -391,16 +405,16 @@ prep_invoke_args(tMacro * pMac)
  *      helpful, but not crucial.  Please contact the author if you think you
  *      might actually want to use this.
 =*/
-tMacro *
-mFunc_Debug(tTemplate* pT, tMacro* pMac)
+macro_t *
+mFunc_Debug(templ_t * pT, macro_t * mac)
 {
     static int dummy = 0;
-    char const * pz  = pT->pzTemplText + pMac->ozText;
-    int  for_index = (forInfo.fi_depth <= 0)
+    char const * pz  = pT->td_text + mac->md_txt_off;
+    int  for_index = (curr_ivk_info->ii_for_depth <= 0)
         ? -1
-        : forInfo.fi_data[ forInfo.fi_depth - 1].for_index;
+        : curr_ivk_info->ii_for_data[ curr_ivk_info->ii_for_depth-1].for_index;
 
-    fprintf(pfTrace, FN_DEBUG, pz, for_index);
+    fprintf(trace_fp, FN_DEBUG, pz, for_index);
 
     /*
      *  The case element values were chosen to thwart most
@@ -433,46 +447,46 @@ mFunc_Debug(tTemplate* pT, tMacro* pMac)
     default:   dummy++;
     }
     if (IS_GRAPHIC_CHAR(dummy))
-        fprintf(pfTrace, FN_DEBUG_GRAPHIC, dummy);
-    putc(NL, pfTrace);
-    return pMac+1;
+        fprintf(trace_fp, FN_DEBUG_GRAPHIC, dummy);
+    putc(NL, trace_fp);
+    return mac+1;
 }
 
-
-/*
- *  build_defs
+/**
+ *  Build up a definition context.
+ *  It is created by passed-in macro arguments.
  *
- *  Build up a definition context created by passed-in macro arguments
+ * @param def_ct    number of definitions
+ * @param def_list  list of definitions
  */
 static void
-build_defs(int defCt, tDefList* pList)
+build_defs(int def_ct, def_list_t * def_list)
 {
-    tDefEntry* pDefs = &(pList->de);
-    currDefCtx.pDefs = pDefs;
+    curr_def_ctx.dcx_defent = &(def_list->de);
 
     /*
      *  FOR each definition, evaluate the associated expression
      *      and set the text value to it.
      */
     do  {
-        if (pList->pzExpr == NULL)
+        if (def_list->pzExpr == NULL)
             continue;
 
     retryExpression:
-        switch (*(pList->pzExpr)) {
+        switch (*(def_list->pzExpr)) {
         case ';':
         {
-            char* pz = strchr(pList->pzExpr, NL);
+            char* pz = strchr(def_list->pzExpr, NL);
             if (pz != NULL) {
                 pz = SPN_WHITESPACE_CHARS(pz + 1);
-                pList->pzExpr = pz;
+                def_list->pzExpr = pz;
                 goto retryExpression;
             }
             /* FALLTHROUGH */
         }
         case NUL:
-            pList->pzExpr = NULL;
-            pList->de.val.pzText = (char*)zNil;
+            def_list->pzExpr = NULL;
+            def_list->de.de_val.dvu_text = (char*)zNil;
             break;
 
         case '(':
@@ -484,37 +498,36 @@ build_defs(int defCt, tDefList* pList)
              *  and number results.
              */
             if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
-                fprintf(pfTrace, TRACE_BUILD_DEFS,
-                        pCurMacro->sibIndex - defCt, pList->pzExpr);
+                fprintf(trace_fp, TRACE_BUILD_DEFS,
+                        cur_macro->md_sib_idx - def_ct, def_list->pzExpr);
             }
 
-            res = ag_eval(pList->pzExpr);
+            res = ag_eval(def_list->pzExpr);
             if (AG_SCM_STRING_P(res)) {
-                AGDUPSTR(pList->de.val.pzText, ag_scm2zchars(res, "eval res"),
-                         "dup eval res");
+                AGDUPSTR(def_list->de.de_val.dvu_text,
+                         ag_scm2zchars(res, "res"), "ev res");
             }
             else if (AG_SCM_NUM_P(res)) {
-                pList->de.val.pzText = AGALOC(16, "number buf");
-                snprintf(pList->de.val.pzText, (size_t)16, BUILD_DEFS_LONG_FMT,
-                         AG_SCM_TO_ULONG(res));
+                def_list->de.de_val.dvu_text = AGALOC(16, "num buf");
+                snprintf(def_list->de.de_val.dvu_text, (size_t)16,
+                         BUILD_DEFS_LONG_FMT, AG_SCM_TO_ULONG(res));
             }
             else
-                AGDUPSTR(pList->de.val.pzText, zNil, "empty str");
+                AGDUPSTR(def_list->de.de_val.dvu_text, zNil, "nil str");
             break;
         }
 
         case '`':
             if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
-                fprintf(pfTrace, TRACE_BUILD_DEFS,
-                        pCurMacro->sibIndex - defCt, pList->pzExpr+1);
+                fprintf(trace_fp, TRACE_BUILD_DEFS,
+                        cur_macro->md_sib_idx - def_ct, def_list->pzExpr+1);
             }
-            pList->de.val.pzText = shell_cmd(pList->pzExpr+1);
+            def_list->de.de_val.dvu_text = shell_cmd(def_list->pzExpr+1);
             break;
         }
-    } while (pList++, --defCt > 0);
+    } while (def_list++, --def_ct > 0);
 }
 
-
 /*=macfunc DEFINE
  *
  *  what:    Define a user AutoGen macro
@@ -588,71 +601,69 @@ build_defs(int defCt, tDefList* pList)
  *    This macro ends the @code{DEFINE} function template block.
  *    For a complete description @xref{DEFINE}.
 =*/
-/*
- *  mFunc_Define
+/**
+ *  This routine runs the invocation of a defined macro.
  *
- *  This routine runs the invocation.
+ * @param tpl not used
+ * @param[in] mac the macro that holds the data for the defined macro
  */
-tMacro*
-mFunc_Define(tTemplate* pT, tMacro* pMac)
+macro_t *
+mFunc_Define(templ_t * tpl, macro_t * mac)
 {
-    tDefList*   pList  = (tDefList*)pMac->res;
-    int         defCt  = pMac->sibIndex;
-    tDefCtx     ctx;
-
-    pT = (tTemplate*)pMac->funcPrivate;
+    def_list_t * pList  = (def_list_t *)mac->md_res;
+    int          defCt  = mac->md_sib_idx;
+    def_ctx_t    ctx;
 
     if (OPT_VALUE_TRACE > TRACE_NOTHING) {
-        fprintf(pfTrace, TPL_INVOKED, pT->pzTplName, defCt);
+        tpl = (templ_t*)mac->md_pvt;
+
+        fprintf(trace_fp, TPL_INVOKED, tpl->td_name, defCt);
         if (OPT_VALUE_TRACE == TRACE_EVERYTHING)
-            fprintf(pfTrace, TAB_FILE_LINE_FMT,
-                    pCurTemplate->pzTplFile, pMac->lineNo);
+            fprintf(trace_fp, TAB_FILE_LINE_FMT,
+                    current_tpl->td_file, mac->md_line);
     }
 
     /*
      *  IF we have no special definitions, then do not nest definitions
      */
     if (defCt != 0) {
-        ctx  = currDefCtx;
-        currDefCtx.pPrev = &ctx;
+        ctx  = curr_def_ctx;
+        curr_def_ctx.dcx_prev = &ctx;
         build_defs(defCt, pList);
     }
 
-    {
-        tTemplate*  pOldTpl = pCurTemplate;
-        pCurTemplate = pT;
-
-        generateBlock(pT, pT->aMacros, pT->aMacros + pT->macroCt);
-        pCurTemplate = pOldTpl;
-    }
+    gen_new_block((templ_t*)mac->md_pvt);
 
     if (defCt != 0)
-        currDefCtx = ctx;
+        curr_def_ctx = ctx;
 
-    if ((defCt = pMac->sibIndex) > 0) {
-        pList = (tDefList*)pMac->res;
+    if ((defCt = mac->md_sib_idx) > 0) {
+        pList = (def_list_t *)mac->md_res;
         while (defCt-- > 0) {
             if (pList->pzExpr != NULL) {
-                AGFREE((void*)pList->de.val.pzText);
-                pList->de.val.pzText = NULL;
+                AGFREE((void*)pList->de.de_val.dvu_text);
+                pList->de.de_val.dvu_text = NULL;
             }
             pList++;
         }
     }
 
-    return pMac+1;
+    return mac+1;
 }
 
-
+/**
+ * unload a defined macro
+ *
+ * @param[in,out] mac macro containing the data to unload
+ */
 void
-mUnload_Define(tMacro* pMac)
+mUnload_Define(macro_t * mac)
 {
-    void* p = (void*)(pMac->res);
+    void * p = (void*)(mac->md_res);
     if (p != NULL)
         AGFREE(p);
 }
 
-
 /*=macfunc INVOKE
  *
  *  handler_proc:
@@ -678,37 +689,32 @@ mUnload_Define(tMacro* pMac)
  *  the first time the macro is processed and evaluated again every
  *  time the macro is evaluated.
 =*/
-tMacro *
-mFunc_Invoke(tTemplate * pT, tMacro * pMac)
+macro_t *
+mFunc_Invoke(templ_t * tpl, macro_t * mac)
 {
-    char* pzText;
-    SCM   macName;
-    tTemplate* pInv;
-
     /*
      *  IF this is the first time through,
      *  THEN separate the name from the rest of the arguments.
      */
-    if (pMac->ozName == 0) {
-        prep_invoke_args(pMac);
+    if (mac->md_name_off == 0) {
+        prep_invoke_args(mac);
 
         /*
          *  IF the name is constant and not an expression,
          *  THEN go find the template now and bind the macro call
          *       to a particular template macro
          */
-        if (IS_VAR_FIRST_CHAR(pT->pzTemplText[ pMac->ozName ])) {
-            pMac->funcCode    = FTYP_DEFINE;
-            pMac->funcPrivate = (void*)findTemplate(
-                pT->pzTemplText + pMac->ozName );
+        if (IS_VAR_FIRST_CHAR(tpl->td_text[ mac->md_name_off ])) {
+            mac->md_code = FTYP_DEFINE;
+            mac->md_pvt  = (void*)find_tpl(tpl->td_text + mac->md_name_off);
 
-            if (pMac->funcPrivate == NULL) {
-                pzText = aprf(BAD_MAC_NM_FMT, pT->pzTemplText + pMac->ozName);
-                AG_ABEND_IN(pT, pMac, pzText);
+            if (mac->md_pvt == NULL) {
+                char const * p = tpl->td_text + mac->md_name_off;
+                AG_ABEND_IN(tpl, mac, aprf(BAD_MAC_NM_FMT, p));
                 /* NOTREACHED */
             }
 
-            return mFunc_Define(pT, pMac);
+            return mFunc_Define(tpl, mac);
         }
     }
 
@@ -716,19 +722,19 @@ mFunc_Invoke(tTemplate * pT, tMacro * pMac)
      *  Call `eval' to determine the macro name.  Every invocation
      *  may be different!!
      */
-    macName = eval(pT->pzTemplText + pMac->ozName);
+    {
+        SCM    macName = eval(tpl->td_text + mac->md_name_off);
+        char * pz = ag_scm2zchars(macName, "mac name");
+        templ_t * ntpl = find_tpl(pz);
+        if (ntpl == NULL) {
+            pz = aprf(BAD_MAC_NM_FMT, pz);
+            AG_ABEND_IN(tpl, mac, pz);
+            /* NOTREACHED */
+        }
 
-    pzText = ag_scm2zchars(macName, "macro name");
-    pInv = findTemplate(pzText);
-    if (pInv == NULL) {
-        pzText = aprf(BAD_MAC_NM_FMT, pzText);
-        AG_ABEND_IN(pT, pMac, pzText);
-        /* NOTREACHED */
+        mac->md_pvt = (void*)ntpl;
     }
-
-    pMac->funcPrivate = (void*)pInv;
-
-    return mFunc_Define(pT, pMac);
+    return mFunc_Define(tpl, mac);
 }
 
 /**
@@ -738,124 +744,133 @@ mFunc_Invoke(tTemplate * pT, tMacro * pMac)
  * @param ppzSan pointer to scanning pointer
  * @returns      the next open macro slot
  */
-tMacro *
-mLoad_Debug(tTemplate * pT, tMacro * pMac, char const ** ppzScan)
+macro_t *
+mLoad_Debug(templ_t * pT, macro_t * pMac, char const ** ppzScan)
 {
     if (OPT_VALUE_TRACE >= TRACE_DEBUG_MESSAGE)
         return mLoad_Unknown(pT, pMac, ppzScan);
     return mLoad_Comment(pT, pMac, ppzScan);
 }
 
-tMacro *
-mLoad_Define(tTemplate * pT, tMacro * pMac, char const ** ppzScan)
+static templ_t *
+new_template(templ_t * base_tpl, macro_t * mac, char const * scan)
 {
-    char *       pzCopy;             /* next text dest   */
-    tTemplate *  pNewT;
+    templ_t * ntpl;
+    char * copy;
+    char const * src = (char const *)mac->md_txt_off;
+    int ct = base_tpl->td_mac_ct - (mac - base_tpl->td_macros);
+    size_t aloc_sz =
+        sizeof(*ntpl) + (ct * sizeof(macro_t)) + strlen(scan) + 0x100;
+    aloc_sz &= ~0x0F;
 
     /*
+     *  Allocate a new template block that is much larger than needed.
+     */
+    ntpl = (templ_t*)AGALOC(aloc_sz, "new tpl");
+    memset((void*)ntpl, 0, aloc_sz);
+    ntpl->td_magic  = base_tpl->td_magic;
+    ntpl->td_size   = aloc_sz;
+    ntpl->td_mac_ct = ct;
+    ntpl->td_file   = strdup(base_tpl->td_file);
+
+    copy = ntpl->td_name = (void*)(ntpl->td_macros + ct);
+    if (! IS_VAR_FIRST_CHAR(*src))
+        AG_ABEND_IN(base_tpl, mac, LD_DEF_NEED_NAME);
+
+    while (IS_VALUE_NAME_CHAR(*src))
+        *(copy++) = *(src++);
+
+    *(copy++) = NUL;
+
+    if (OPT_VALUE_TRACE >= TRACE_BLOCK_MACROS)
+        fprintf(trace_fp, TRACE_MACRO_DEF, ntpl->td_name, ntpl->td_file);
+
+    ntpl->td_text = copy;
+    ntpl->td_scan = copy+1;
+    strcpy(ntpl->td_start_mac, base_tpl->td_start_mac);
+    strcpy(ntpl->td_end_mac,   base_tpl->td_end_mac);
+    current_tpl = ntpl;
+
+    return ntpl;
+}
+
+static void
+load_define_tpl(templ_t * tpl, char const ** ppzScan)
+{
+    macro_t * last_mac = parse_tpl(tpl->td_macros, ppzScan);
+    int     ct;
+
+    /*
+     *  Make sure all of the input string was *NOT* scanned.
+     */
+    if (*ppzScan == NULL)
+        AG_ABEND_IN(tpl, tpl->td_macros, LD_DEF_WOOPS);
+
+    ct = last_mac - tpl->td_macros;
+
+    /*
+     *  IF there are empty macro slots,
+     *  THEN pack the text
+     */
+    if (ct < tpl->td_mac_ct) {
+        int    delta = sizeof(macro_t) * (tpl->td_mac_ct - ct);
+        void * data  = (tpl->td_name == NULL) ? tpl->td_text : tpl->td_name;
+        size_t size  = tpl->td_scan - (char*)data;
+        memmove((void*)last_mac, data, size);
+
+        tpl->td_text  -= delta;
+        tpl->td_scan  -= delta;
+        tpl->td_name  -= delta;
+        tpl->td_mac_ct = ct;
+    }
+}
+
+macro_t *
+mLoad_Define(templ_t * ori_tpl, macro_t * mac, char const ** p_scan)
+{
+    static load_proc_p_t apDefineLoad[ FUNC_CT ] = { NULL };
+
+    templ_t * new_tpl;
+
+    /**
      *  Save the global macro loading mode
      */
-    tpLoadProc const * papLP = papLoadProc;
-    static tpLoadProc apDefineLoad[ FUNC_CT ] = { NULL };
+    load_proc_p_t const * save_load_procs = load_proc_table;
 
-    if (pMac->ozText == 0)
-        AG_ABEND_IN(pT, pMac, LD_DEF_NEED_NAME);
+    if (mac->md_txt_off == 0)
+        AG_ABEND_IN(ori_tpl, mac, LD_DEF_NEED_NAME);
 
     /*
-     *  IF this is the first time here,
-     *  THEN set up the "DEFINE" block callout table.
-     *  It is the standard table, except entries are inserted
-     *  for functions that are enabled only while processing
+     *  IF this is the first time here, THEN set up the "DEFINE" block
+     *  callout table.  It is the standard table, except entries are
+     *  inserted for functions that are enabled only while processing
      *  a DEFINE block (viz. "ENDDEF" and removing "DEFINE").
      */
     if (apDefineLoad[0] == NULL) {
-        memcpy((void*)apDefineLoad, apLoadProc, sizeof(apLoadProc));
+        memcpy((void*)apDefineLoad, base_load_table, sizeof(base_load_table));
         apDefineLoad[ FTYP_ENDDEF ] = &mLoad_Ending;
         apDefineLoad[ FTYP_DEFINE ] = &mLoad_Bogus;
     }
-    papLoadProc = apDefineLoad;
 
-    {
-        char const*    pzScan = *ppzScan;  /* text after macro */
-        char const*    pzSrc  = (char const*)pMac->ozText; /* macro text */
-        int            macCt  = pT->macroCt - (pMac - pT->aMacros);
-
-        size_t alocSize = sizeof(*pNewT)
-            + (macCt * sizeof(tMacro)) + strlen(pzScan) + 0x100;
-        alocSize &= ~0x0F;
-
-        /*
-         *  Allocate a new template block that is much larger than needed.
-         */
-        pNewT = (tTemplate*)AGALOC(alocSize, "AG macro def");
-        memset((void*)pNewT, 0, alocSize);
-        pNewT->magic      = pT->magic;
-        pNewT->descSize   = alocSize;
-        pNewT->macroCt    = macCt;
-        pNewT->pzTplFile  = strdup(pT->pzTplFile);
-
-        pzCopy = pNewT->pzTplName = (void*)(pNewT->aMacros + macCt);
-        if (! IS_VAR_FIRST_CHAR(*pzSrc))
-            AG_ABEND_IN(pT, pMac, LD_DEF_NEED_NAME);
-
-        while (IS_VALUE_NAME_CHAR(*pzSrc))
-            *(pzCopy++) = *(pzSrc++);
-    }
-
-    if (OPT_VALUE_TRACE >= TRACE_BLOCK_MACROS)
-        fprintf(pfTrace, TRACE_MACRO_DEF,
-                pNewT->pzTplName, pNewT->pzTplFile);
-
-    *(pzCopy++) = NUL;
-    pNewT->pzTemplText = pzCopy;
-    pNewT->pNext = pzCopy+1;
-    strcpy(pNewT->zStartMac, pT->zStartMac);
-    strcpy(pNewT->zEndMac, pT->zEndMac);
-    pCurTemplate = pNewT;
-
-    {
-        tMacro* pMacEnd = parseTemplate(pNewT->aMacros, ppzScan);
-        int     ct;
-
-        /*
-         *  Make sure all of the input string was *NOT* scanned.
-         */
-        if (*ppzScan == NULL)
-            AG_ABEND_IN(pNewT, pNewT->aMacros, LD_DEF_WOOPS);
-
-        ct = pMacEnd - pNewT->aMacros;
-
-        /*
-         *  IF there are empty macro slots,
-         *  THEN pack the text
-         */
-        if (ct < pNewT->macroCt) {
-            int   delta = sizeof(tMacro) * (pNewT->macroCt - ct);
-            void* data  = (pNewT->pzTplName == NULL) ?
-                pNewT->pzTemplText : pNewT->pzTplName;
-            size_t size = pNewT->pNext - (char*)data;
-            memmove((void*)pMacEnd, data, size);
-
-            pNewT->pzTemplText -= delta;
-            pNewT->pNext       -= delta;
-            pNewT->pzTplName   -= delta;
-            pNewT->macroCt      = ct;
-        }
-    }
+    load_proc_table = apDefineLoad;
+    defining_macro  = true;
+    new_tpl = new_template(ori_tpl, mac, *p_scan);
+    load_define_tpl(new_tpl, p_scan);
+    defining_macro  = false;
 
     /*
      *  Adjust the sizes.  Remove absolute pointers.  Reallocate to the correct
      *  size.  Restore the offsets to pointer values.
      */
     {
-        size_t sz = pNewT->pNext - (char*)pNewT;
-        if (sz < pNewT->descSize) {
-            pNewT->descSize     = sz;
-            pNewT->pzTplName   -= (long)pNewT;
-            pNewT->pzTemplText -= (long)pNewT;
-            pNewT = AGREALOC((void*)pNewT, pNewT->descSize, "resize AG mac");
-            pNewT->pzTplName   += (long)pNewT;
-            pNewT->pzTemplText += (long)pNewT;
+        size_t sz = new_tpl->td_scan - (char*)new_tpl;
+        if (sz < new_tpl->td_size) {
+            new_tpl->td_size = sz;
+            new_tpl->td_name -= (long)new_tpl;
+            new_tpl->td_text -= (long)new_tpl;
+            new_tpl = AGREALOC((void*)new_tpl, new_tpl->td_size, "resize mac");
+            new_tpl->td_name += (long)new_tpl;
+            new_tpl->td_text += (long)new_tpl;
         }
     }
 
@@ -863,17 +878,17 @@ mLoad_Define(tTemplate * pT, tMacro * pMac, char const ** ppzScan)
     if (HAVE_OPT(SHOW_DEFS)) {
         static char const zSum[] = "loaded %d macros from %s\n"
             "\tBinary template size:  0x%X\n\n";
-        fprintf(pfTrace, zSum, pNewT->macroCt, pNewT->pzTplFile,
-                (unsigned int)pNewT->descSize);
+        fprintf(trace_fp, zSum, new_tpl->td_mac_ct, new_tpl->td_file,
+                (unsigned int)new_tpl->td_size);
     }
 #endif
 
-    pNewT->pNext  = (char*)pNamedTplList;
-    pNamedTplList = pNewT;
-    papLoadProc   = papLP;
-    memset((void*)pMac, 0, sizeof(*pMac));
-    pCurTemplate  = pT;
-    return pMac;
+    new_tpl->td_scan = (char*)named_tpls;
+    named_tpls      = new_tpl;
+    load_proc_table = save_load_procs;
+    memset((void*)mac, 0, sizeof(*mac));
+    current_tpl     = ori_tpl;
+    return mac;
 }
 /*
  * Local Variables:
